@@ -136,7 +136,8 @@ class Dispatch(Protocol, Generic[State, Action, Event]):
     def __call__(
         self: Dispatch,
         *items: Action | Event | list[Action | Event],
-        with_state: Callable[[State | None], Action | Event | list[Action | Event]],
+        with_state: Callable[[State | None], Action | Event | list[Action | Event]]
+        | None = None,
     ) -> None:
         ...
 
